<?php
/**
 * AWeber API Handler
 *
 * @package     RCP\AWeberPro\API
 * @since       1.0.0
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * AWeber API handler class
 *
 * @since       1.0.0
 */
class RCP_AWeber_Pro_API {


	/**
	 * @var         object $api The AWeber API object
	 * @since       1.0.0
	 */
	public $api;


	/**
	 * @var         array $settings AWeber Pro settings
	 * @since       1.0.0
	 */
	public $settings;


	/**
	 * @var         array $lists Per-level list settings
	 * @since       1.0.0
	 */
	public $lists;


	/**
	 * Get things started
	 *
	 * @access      public
	 * @since       1.0.0
	 * @return      void
	 */
	public function __construct() {
		$this->setup_api();

		// Maybe signup a user
		add_action( 'rcp_form_processing', array( $this, 'maybe_signup' ), 10, 2 );
	}


	/**
	 * Setup the API object
	 *
	 * @access      private
	 * @since       1.0.0
	 * @return      void
	 */
	private function setup_api() {
		if( ! $this->api ) {
			$this->settings = get_option( 'rcp_aweber_pro_settings' );
			$this->lists    = get_option( 'rcp_aweber_pro_subscription_lists' );

			if( ! empty( $this->settings['auth_code'] ) ) {
				$return     = '';
				$error_code = '';
				$secrets    = get_option( 'rcp_aweber_pro_secrets' );

				if( $secrets !== false ) {
					try {
						$this->api = new AWeberAPI( $secrets['consumer_key'], $secrets['consumer_secret'] );

						$return = __( 'Connected', 'rcp-aweber-pro' );
					} catch( AWeberAPIException $e ) {
						$this->api = false;
					}
				} else {
					try {
						list( $consumer_key, $consumer_secret, $access_key, $access_secret ) = AWeberAPI::getDataFromAweberID( $this->settings['auth_code'] );
					} catch( AWeberAPIException $e ) {
						list( $consumer_key, $consumer_secret, $access_key, $access_secret ) = null;

						$desc = $e->getMessage();
						$desc = preg_replace( '/http.*$/i', '', $desc );
						$desc = preg_replace( '/[\.\!:]+.*$/i', '', $desc );

						$error_code = ' (' . $desc . ')';
					} catch( AWeberOAuthDataMissing $e ) {
						list( $consumer_key, $consumer_secret, $access_key, $access_secret ) = null;
					} catch( AWeberException $e ) {
						list( $consumer_key, $consumer_secret, $access_key, $access_secret ) = null;
					}

					if( ! $access_secret ) {
						$return = sprintf( __( 'Unable to connect to your AWeber account%s<br />', 'rcp-aweber-pro' ), $error_code );

						# Show oauth_id if it failed and an api exception was not raised
						if( empty( $error_code ) ) {
							$return .= sprintf( __( 'The provided authorization code was: %s<br />', 'rcp-aweber-pro' ), $this->settings['auth_code'] );
						}

						$return .= __( 'Please verify your authorization code and try again.', 'rcp-aweber-pro' );
					} else {
						$secrets = array(
							'consumer_key'    => $consumer_key,
							'consumer_secret' => $consumer_secret,
							'access_key'      => $access_key,
							'access_secret'   => $access_secret
						);

						update_option( 'rcp_aweber_pro_secrets', $secrets );

						$return = __( 'Connected', 'rcp-aweber-pro' );
					}
				}
			} else {
				delete_option( 'rcp_aweber_pro_secrets' );
			}

			$return = ! empty( $return ) ? $return : __( 'Not Connected', 'rcp-aweber-pro' );

			update_option( 'rcp_aweber_pro_status', $return );
		}
	}


	/**
	 * Retrieve the list to signup for
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       int $level_id The ID of the subscription level to lookup
	 * @return      string $list The AWeber list to signup for
	 */
	public function get_list( $level_id ) {
		if( is_array( $this->lists ) && array_key_exists( $level_id, $this->lists ) && $this->lists[$level_id] !== 'inherit' ) {
			$list = $this->lists[$level_id];
		} else {
			$list = $this->settings['saved_list'];
		}

		return apply_filters( 'rcp_aweber_pro_get_list', $list, $level_id );
	}


	/**
	 * Check if a given member is subscribed on AWeber
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       string $email The email address to lookup
	 * @param       string $list The list to check
	 * @return      bool $member_exists True if subscribed, false otherwise
	 */
	public function member_exists( $email, $list ) {
		$secrets = get_option( 'rcp_aweber_pro_secrets' );

		try {
			$account     = $this->api->getAccount( $secrets['access_key'], $secrets['access_secret'] );
			$list_url    = '/accounts/' . $account->id . '/lists/4156144';
			$list        = $account->loadFromUrl( $list_url );
			$subscribers = $list->subscribers;

			if( $subscribers->data['total_size'] > 0 ) {
				foreach( $subscribers->data['entries'] as $subscriber ) {
					if( $subscriber['email'] == $email ) {
						$member_exists = true;
						break;
					}
				}
			}
		} catch( Exception $e ) {
			$member_exists = false;
		}

		return $member_exists;
	}


	/**
	 * Subscribe an email to AWeber
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       string $email The email address to subscribe
	 * @param       int $user_id The ID of the user to subscribe
	 * @return      bool True if added successfully, false otherwise
	 */
	public function subscribe( $email, $user_id ) {
		$return     = false;
		$secrets    = get_option( 'rcp_aweber_pro_secrets' );
		$saved_list = $this->settings['saved_list'];
		$level_list = $this->get_list( $_POST['rcp_level'] );
		$fname      = isset( $_POST['rcp_user_first'] ) ? sanitize_text_field( $_POST['rcp_user_first'] ) : '';
		$lname      = isset( $_POST['rcp_user_last'] ) ? sanitize_text_field( $_POST['rcp_user_last'] ) : '';
		$status     = rcp_get_status( $user_id );

		if( $fname ) {
			$name = $fname;

			if( $lname ) {
				$name .= ' ' . $lname;
			}
		} elseif( $lname ) {
			$name = $lname;
		} else {
			$name = $email;
		}

		$params = array( 'email' => $email, 'name' => $name, 'ip_address' => rcp_get_ip() );

		try {
			$account  = $this->api->getAccount( $secrets['access_key'], $secrets['access_secret'] );
			$list_url = '/accounts/' . $account->id . '/lists/' . $level_list;
			$list     = $account->loadFromUrl( $list_url );

			$subscribers    = $list->subscribers;
			$new_subscriber = $subscribers->create( $params );
		} catch( Exception $e ) {
			$return = false;
		}

		if( $return === false ) {
			return false;
		} else {
			if( isset( $this->settings['double_subscribe'] ) && $saved_list != $level_list ) {
				try {
					$account  = $this->api->getAccount( $secrets['access_key'], $secrets['access_secret'] );
					$list_url = '/accounts/' . $account->id . '/lists/' . $level_list;
					$list     = $account->loadFromUrl( $list_url );

					$subscribers    = $list->subscribers;
					$new_subscriber = $subscribers->create( $params );
				} catch( Exception $e ) {
					$return = false;
				}
			}

			if( $return === false ) {
				return false;
			} else {
				return true;
			}
		}
	}


	/**
	 * Maybe sign up a given user
	 *
	 * @access      public
	 * @since       1.0.0
	 * @param       array $posted The fields posted by the submission form
	 * @param       int $user_id The ID of this user
	 * @return      void
	 */
	public function maybe_signup( $posted, $user_id ) {
		$email = false;

		if( isset( $posted['rcp_aweber_pro_signup'] ) ) {
			if( is_user_logged_in() ) {
				$user_data = get_userdata( $user_id );
				$email     = $user_data->user_email;
			} else {
				$email = $posted['rcp_user_email'];
			}
		}

		if( $email ) {
			$subscribed = $this->subscribe( $email, $user_id );

			if( $subscribed ) {
				update_user_meta( $user_id, 'rcp_subscribed_to_aweber', 'yes' );
			}
		}
	}
}
