<?php
/**
 * Plugin Name:     Restrict Content Pro - AWeber Pro
 * Plugin URI:      https://restrictcontentpro.com/downloads/aweberpro
 * Description:     Include a AWeber signup option with your RCP registration form
 * Version:         1.1.2
 * Author:          Restrict Content Pro Team
 * Text Domain:     rcp-aweber-pro
 *
 * @package         RCP\AWeberPro
 * @author          Daniel J Griffiths <dgriffiths@section214.com>
 * @copyright       Copyright (c) 2015, Daniel J Griffiths
 */


// Exit if accessed directly
if( ! defined( 'ABSPATH' ) ) {
	exit;
}


if( ! class_exists( 'RCP_AWeber_Pro' ) ) {


	/**
	 * Main RCP_AWeber_Pro class
	 *
	 * @since       1.0.0
	 */
	class RCP_AWeber_Pro {


		/**
		 * @var         RCP_AWeber_Pro $instance The one true RCP_AWeber_Pro
		 * @since       1.0.0
		 */
		private static $instance;


		/**
		 * @var         object $api_helper The AWeber API helper object
		 * @since       1.0.0
		 */
		public $api_helper;


		/**
		 * Get active instance
		 *
		 * @access      public
		 * @since       1.0.0
		 * @return      object self::$instance The one true RCP_AWeber_Pro
		 */
		public static function instance() {
			if( ! self::$instance ) {
				self::$instance = new RCP_AWeber_Pro();
				self::$instance->setup_constants();
				self::$instance->includes();
				self::$instance->load_textdomain();
				self::$instance->hooks();
				self::$instance->api_helper = new RCP_AWeber_Pro_API();
			}

			return self::$instance;
		}


		/**
		 * Setup plugin constants
		 *
		 * @access      private
		 * @since       1.0.0
		 * @return      void
		 */
		public function setup_constants() {
			// Plugin version
			define( 'RCP_AWEBER_PRO_VER', '1.1.2' );

			// Plugin path
			define( 'RCP_AWEBER_PRO_DIR', plugin_dir_path( __FILE__ ) );

			// Plugin URL
			define( 'RCP_AWEBER_PRO_URL', plugin_dir_url( __FILE__ ) );
		}


		/**
		 * Include necessary files
		 *
		 * @access      private
		 * @since       1.0.0
		 * @return      void
		 */
		private function includes() {
			require_once RCP_AWEBER_PRO_DIR . 'includes/functions.php';
			require_once RCP_AWEBER_PRO_DIR . 'includes/libraries/AWeber/aweber_api/aweber_api.php';
			require_once RCP_AWEBER_PRO_DIR . 'includes/class.aweber-api.php';

			if( is_admin() ) {
				require_once RCP_AWEBER_PRO_DIR . 'includes/admin/settings/register.php';
			}
		}


		/**
		 * Run action and filter hooks
		 *
		 * @access      private
		 * @since       1.0.0
		 * @return      void
		 */
		private function hooks() {
			// Add the subscription checkbox
			add_action( 'rcp_before_registration_submit_field', array( $this, 'add_fields' ), 100 );

			// Display the signed up notice
			add_action( 'rcp_edit_member_after', array( $this, 'display_signup_notice' ) );

			// Plugin updates
			if( class_exists( 'RCP_Add_On_Updater' ) ) {
				$updater = new RCP_Add_On_Updater( 306, __FILE__, RCP_AWEBER_PRO_VER );
			}
		}


		/**
		 * Internationalization
		 *
		 * @access      public
		 * @since       1.0.0
		 * @return      void
		 */
		public static function load_textdomain() {
			// Set filter for language directory
			$lang_dir = dirname( plugin_basename( __FILE__ ) ) . '/languages/';
			$lang_dir = apply_filters( 'rcp_aweber_pro_languages_directory', $lang_dir );

			// Traditional WordPress plugin locale filter
			$locale = apply_filters( 'plugin_locale', get_locale(), 'rcp-aweber-pro' );
			$mofile = sprintf( '%1$s-%2$s.mo', 'rcp-aweber-pro', $locale );

			// Setup paths to current locale file
			$mofile_local   = $lang_dir . $mofile;
			$mofile_global  = WP_LANG_DIR . '/rcp-aweber-pro/' . $mofile;

			if( file_exists( $mofile_global ) ) {
				// Look in global /wp-content/languages/rcp-aweber-pro/ folder
				load_textdomain( 'rcp-aweber-pro', $mofile_global );
			} elseif( file_exists( $mofile_local ) ) {
				// Look in local /wp-content/plugins/rcp-aweber-pro/languages/ folder
				load_textdomain( 'rcp-aweber-pro', $mofile_local );
			} else {
				// Load the default language files
				load_plugin_textdomain( 'rcp-aweber-pro', false, $lang_dir );
			}
		}


		/**
		 * Add the subscription field to the registration form
		 *
		 * @access      public
		 * @since       1.0.0
		 * @return      void
		 */
		public function add_fields() {
			ob_start();
			if( rcp_aweber_pro_show_checkbox() ) {
				$settings = get_option( 'rcp_aweber_pro_settings' );

				if( isset( $settings['auto_subscribe'] ) ) {
					echo '<input id="rcp_aweber_pro_signup" name="rcp_aweber_pro_signup" type="hidden" value="true" />';
				} else {
					echo '<p>';
					echo '<input id="rcp_aweber_pro_signup" name="rcp_aweber_pro_signup" type="checkbox" checked="checked" />';
					echo '<label for="rcp_aweber_pro_signup">' . ( isset( $settings['signup_label'] ) && ! empty( $settings['signup_label'] ) ? $settings['signup_label'] : __( 'Signup for Newsletter', 'rcp-aweber-pro' ) ) . '</label>';
					echo '</p>';
				}
			}
			echo ob_get_clean();
		}


		/**
		 * Display a signed up notice if user has subscribed
		 *
		 * @access      public
		 * @since       1.0.0
		 * @param       int $user_id The user ID of this user
		 * @return      void
		 */
		public function display_signup_notice( $user_id ) {
			$signed_up = get_user_meta( $user_id, 'rcp_subscribed_to_aweber', true );
			$signed_up = ( $signed_up ? __( 'Yes', 'rcp-aweber-pro' ) : __( 'No', 'rcp-aweber-pro' ) );

			echo '<tr class="form-field">';
			echo '<th scope="row" valign="top">' . __( 'AWeber', 'rcp-aweber-pro' ) . '</th>';
			echo '<td>' . $signed_up . '</td>';
			echo '</tr>';
		}
	}
}


/**
 * The main function responsible for returning the one true RCP_AWeber_Pro
 * instance to functions everywhere
 *
 * @since       1.0.0
 * @return      RCP_AWeber_Pro The one true RCP_AWeber_Pro
 */
function rcp_aweber_pro() {
    return RCP_AWeber_Pro::instance();
}
add_action( 'plugins_loaded', 'rcp_aweber_pro' );
