<?php
namespace RCP\Addon\DripContent\Shortcodes;

defined( 'WPINC' ) or die;

/**
 * Renders the current member's drip content schedule.
 *
 * @since 1.0
 * @since 1.0.1 Added orderby attribute.
 * @param array $atts {
 * 	Shortcode attributes. Optional.
 * 	@type bool   $show_future Whether or not to show future content not yet available.
 * 	@type string $orderby     The sort order of the schedule. Default 'id'. Options: 'id', 'name', 'date'.
 * }
 * @return string
 */
function member_content_shortcode( $atts ) {

	$atts = shortcode_atts( array(
		'show_future' => true,
		'orderby'     => 'id',
	), $atts, 'rcp_drip_content' );

	if ( 'false' === $atts['show_future'] ) {
		$atts['show_future'] = false;
	}

	if ( ! is_user_logged_in() ) {
		return;
	}

	$user_id = wp_get_current_user()->ID;

	$sub_id = (int) rcp_get_subscription_id( $user_id );

	if ( ! $sub_id ) {
		return;
	}

	$schedules = rcp_drip_get_level_schedules( $sub_id, $atts['orderby'] );

	if ( ! $schedules || empty( $schedules[0] ) ) {
		return;
	}

	// Sort the schedules
	switch ( $atts['orderby'] ) {

		case 'name':

			foreach ( $schedules as $key => $schedule ) {
				if ( 'post' === $schedule['type'] ) {
					$schedules[$key]['name'] = get_post_field( 'post_title', $schedule['post_id'], 'raw' );
				}

				if ( 'taxonomy' === $schedule['type'] ) {
					$schedules[$key]['name'] = get_term_by( 'term_taxonomy_id', $schedule['taxonomy_term'] )->name;
				}
			}

			usort( $schedules, function( $a, $b ) {
				return strcasecmp( $a['name'], $b['name'] );
			});

			break;

		case 'date':
			foreach ( $schedules as $key => $schedule ) {
				$schedules[$key]['time'] = strtotime( $schedule['unit'] . ' ' . $schedule['period'] );
			}

			usort( $schedules, function( $a, $b ) {
				return $a['time'] - $b['time'];
			});

			break;

		case 'id':
		default:
			break;

	}

	$rcp_member = new \RCP_Member( $user_id );

	$timezone = get_option( 'timezone_string' );

	$timezone = ! empty( $timezone ) ? $timezone : 'UTC';

	$join_date = new \DateTime( $rcp_member->get_joined_date( $sub_id ), new \DateTimeZone( $timezone ) );

	$join_date->modify( 'midnight' );

	$now = new \DateTime( NULL, new \DateTimeZone( $timezone ) );

	$output = '<ul class="rcp-drip-available-content">';

	foreach( $schedules as $key => $schedule ) {

		if ( 'post' === $schedule['type'] ) {

			$post_available_date = clone $join_date;

			$post_available_date->modify( $schedule['unit'] . ' ' . $schedule['period'] );

			$post_available_date->modify( 'midnight' );

			$post = get_post( $schedule['post_id'] );

			if ( $now >= $post_available_date ) {

				$output .= '<li><a href="' . esc_url( get_permalink( $post ) ) . '">' . get_the_title( $post ) . '</a></li>';

			} else if ( $atts['show_future'] ) {

				$output .= '<li>' . $post->post_title . ' ' . sprintf( __( '(Available on %s)', 'rcp-drip-content' ), $post_available_date->format( get_option( 'date_format' ) ) ) . '</li>';
			}
		}

		if ( 'taxonomy' === $schedule['type'] ) {

			$taxonomy_available_date = clone $join_date;

			$taxonomy_available_date->modify( $schedule['unit'] . ' ' . $schedule['period'] );

			$term = get_term( $schedule['taxonomy_term'] );

			if ( $now >= $taxonomy_available_date ) {

				$output .= '<li><a href="' . esc_url( get_term_link( $term ) ) . '">' . ucfirst( $term->taxonomy ) . ': ' . $term->name . '</a></li>';

			} else if ( $atts['show_future'] ) {

				$output .= '<li>' . ucfirst( $term->taxonomy ) . ': ' . $term->name . ' ' . sprintf( __( '(Available on %s)', 'rcp-drip-content' ), $taxonomy_available_date->format( get_option( 'date_format' ) ) ) . '</li>';

			}
		}

	}

	$output .= '</ul>';

	return $output;
}
add_shortcode( 'rcp_drip_member_content', '\RCP\Addon\DripContent\Shortcodes\member_content_shortcode' );