<?php
/**
 * Meta box functions
 *
 * @package     RCP\IP_Restriction\MetaBox
 * @since       1.0.0
 */


// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Register meta boxes for IP Restriction
 *
 * @since       1.0.0
 * @return      void
 */
function rcp_ip_restriction_add_meta_boxes() {
	$post_types = get_post_types( array( 'public' => true, 'show_ui' => true ), 'objects' );
	$exclude    = apply_filters( 'rcp_metabox_excluded_post_types', array( 'forum', 'topic', 'reply', 'product' ) );

	foreach ( $post_types as $page ) {
		if ( ! in_array( $page->name, $exclude ) ) {
			add_meta_box( 'rcp-ip-restriction', __( 'IP Restriction Whitelist', 'rcp-ip-restriction' ), 'rcp_ip_restriction_render_meta_box', $page->name, 'side', 'default' );
		}
	}
}
add_action( 'add_meta_boxes', 'rcp_ip_restriction_add_meta_boxes' );


/**
 * Render meta box
 *
 * @since       1.0.0
 * @global      object $post The post we are editing
 * @return      void
 */
function rcp_ip_restriction_render_meta_box() {
	global $post;

	$post_id   = $post->ID;
	$whitelist = get_post_meta( $post_id, '_whitelist', true );
	$whitelist = is_array( $whitelist ) ? $whitelist : array();

	echo '<textarea name="_whitelist" id="_whitelist" style="width: 100%; height: 200px;">' . implode( "\n", $whitelist ) . '</textarea>';
	echo '<label for="_whitelist"><p class="description">' . __( 'Enter IP addresses to whitelist, one per line. Users who match these IP addresses will bypass any restrictions.', 'rcp-ip-restriction' ) . '</p></label>';

	// Allow extension of the IP Restriction meta box
	do_action( 'rcp_ip_restriction_meta_box_fields', $post->ID );

	wp_nonce_field( basename( __FILE__ ), 'rcp_ip_restriction_meta_box_nonce' );
}


/**
 * Save post meta when the save_post action is called
 *
 * @since       1.0.0
 * @param       int $post_id The ID of the post we are saving
 * @global      object $post The post we are saving
 * @return      void
 */
function rcp_ip_restriction_meta_box_save( $post_id ) {
	global $post;

	// Don't process if nonce can't be validated
	if ( ! isset( $_POST['rcp_ip_restriction_meta_box_nonce'] ) || ! wp_verify_nonce( $_POST['rcp_ip_restriction_meta_box_nonce'], basename( __FILE__ ) ) ) {
		return $post_id;
	}

	// Don't process if this is an autosave
	if ( ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) || isset( $_REQUEST['bulk_edit'] ) ) {
		return $post_id;
	}

	// Don't save if user shouldn't be editing this
	if ( 'page' == $_POST['post_type'] ) {
		if ( ! current_user_can( 'edit_page', $post_id ) ) {
			return $post_id;
		}
	} elseif ( ! current_user_can( 'edit_post', $post_id ) ) {
		return $post_id;
	}

	// The default fields that get saved
	$fields = apply_filters( 'rcp_ip_restriction_meta_box_fields_save', array(
		'_whitelist'
	) );

	foreach ( $fields as $field ) {
		if ( isset( $_POST[ $field ] ) ) {
			if ( is_string( $_POST[ $field ] ) ) {
				$new = esc_attr( $_POST[ $field ] );
			} else {
				$new = $_POST[ $field ];
			}

			$new = apply_filters( 'rcp_ip_restriction_meta_box_save_' . $field, $new );

			update_post_meta( $post_id, $field, $new );
		} else {
			delete_post_meta( $post_id, $field );
		}
	}
}
add_action( 'save_post', 'rcp_ip_restriction_meta_box_save' );


/**
 * Sanitize the whitelist on save
 *
 * @since       1.0.0
 * @param       string $new The contents of the whitelist text area
 * @return      array $new The formatted contents of the text area
 */
function rcp_ip_restriction_meta_box_save_whitelist( $new ) {
	// Sanitize the input
	$new = rcp_ip_restriction_sanitize_ips( $new );

	return $new;
}
add_filter( 'rcp_ip_restriction_meta_box_save__whitelist', 'rcp_ip_restriction_meta_box_save_whitelist' );
