<?php
/**
 * Helper functions
 *
 * @package     RCP\IP_Restriction\Functions
 * @since       1.0.0
 */


// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * Sanitize IP fields
 *
 * @since       1.0.0
 * @param       string $ip_addresses Addresses in string format
 * @return      array $ip_addresses Addresses in array format
 */
function rcp_ip_restriction_sanitize_ips( $ip_addresses = '' ) {
	if ( ! is_array( $ip_addresses ) ) {
		$ip_addresses = explode( "\n", $ip_addresses );
	}
	$ip_addresses = array_map( 'trim', $ip_addresses );
	$ip_addresses = array_unique( $ip_addresses );
	$ip_addresses = array_map( 'sanitize_text_field', $ip_addresses );

	return $ip_addresses;
}


/**
 * Check if a given IP address is whitelisted
 *
 * @since       1.0.0
 * @param       int $post_id The ID of a given post
 * @return      bool $is_whitelisted True if whitelisted, false otherwise
 */
function rcp_ip_restriction_is_whitelisted( $post_id ) {
	$is_whitelisted = false;
	$ip_address     = rcp_get_ip();

	// Check global whitelist
	$settings         = get_option( 'rcp_ip_restriction_settings' );
	$global_whitelist = ( isset( $settings['whitelist'] ) ? $settings['whitelist'] : array() );

	foreach ( $global_whitelist as $whitelist_address ) {
		if ( rcp_ip_restriction()->ipcheck->match( $ip_address, $whitelist_address ) ) {
			$is_whitelisted = true;
			break;
		}
	}

	// Maybe check subscription whitelist
	if ( ! $is_whitelisted ) {
		$subscription_whitelists = get_option( 'rcp_ip_restriction_subscription_whitelists' );
		$subscription_levels     = rcp_get_content_subscription_levels( $post_id );

		if ( is_array( $subscription_whitelists ) && is_array( $subscription_levels ) ) {
			foreach ( $subscription_whitelists as $subscription => $whitelist ) {
				if ( in_array( $subscription, $subscription_levels ) ) {
					foreach ( $whitelist as $whitelist_address ) {
						if ( rcp_ip_restriction()->ipcheck->match( $ip_address, $whitelist_address ) ) {
							$is_whitelisted = true;
							break;
						}
					}
				}
			}
		}
	}

	// Maybe check post whitelist
	if ( ! $is_whitelisted ) {
		$post_whitelist = get_post_meta( $post_id, '_whitelist', true );
		$post_whitelist = ( is_array( $post_whitelist ) ? $post_whitelist : array() );

		foreach ( $post_whitelist as $whitelist_address ) {
			if ( rcp_ip_restriction()->ipcheck->match( $ip_address, $whitelist_address ) ) {
				$is_whitelisted = true;
				break;
			}
		}
	}

	return $is_whitelisted;
}


/**
 * Check if a given IP address is blacklisted
 *
 * @since       1.0.0
 * @return      bool $is_blacklisted True if blacklisted, false otherwise
 */
function rcp_ip_restriction_is_blacklisted() {
	$is_blacklisted = false;
	$ip_address     = rcp_get_ip();

	$settings         = get_option( 'rcp_ip_restriction_settings' );
	$global_blacklist = ( isset( $settings['blacklist'] ) ? $settings['blacklist'] : array() );

	foreach ( $global_blacklist as $blacklist_address ) {
		if ( rcp_ip_restriction()->ipcheck->match( $ip_address, $blacklist_address ) ) {
			$is_blacklisted = true;
			break;
		}
	}

	return $is_blacklisted;
}
